# -*- coding: utf-8 -*-
import logging
import socket
import sys

from crashlog.config import conf

from django.views.debug import ExceptionReporter

from .models import Error

logger = logging.getLogger('crashlog')


def process_exception(exception, request=None, message_user=False):
    if exception.__class__ in conf._ignore_exceptions:
        return
    reporter = ExceptionReporter(request, *sys.exc_info())
    server_name = socket.gethostname()
    tb_text = reporter.get_traceback_html()
    class_name = exception.__class__.__name__

    if request:
        url = request.build_absolute_uri()
        user = getattr(request, 'user', None)
    else:
        url = 'N/A'
        user = 'N/A'

    defaults = dict(
        class_name=class_name,
        message=str(exception),
        url=url,
        server_name=server_name,
        traceback=tb_text,
        username=str(user)
    )

    try:
        err = Error.objects.create(**defaults)
        if message_user and request:
            err.message_user(request, str(exception))
        return err
    except Exception as exc:  # pragma: no cover
        logger.critical(exc)


class CrashLogMiddleware(object):

    def __init__(self, get_response=None):
        self.get_response = get_response
        super().__init__()

    def process_exception(self, request, exception):
        process_exception(exception, request)

    def __call__(self, request):
        return self.get_response(request)



import os

import sys

import pytest

from crashlog.models import Error


def pytest_configure(config):

    import warnings
    warnings.simplefilter('once', DeprecationWarning)
    HERE = os.path.join(os.path.dirname(__file__))
    sys.path.insert(0, os.path.join(HERE, 'demo'))



@pytest.fixture()
def error(db):
    return Error.objects.create(class_name='PermissionDenied',
                         message='Error Message',
                         traceback='',
                         url='/',
                         server_name='',
                         )



# -*- coding: utf-8 -*-
import pytest
from django.contrib.auth.models import AnonymousUser, User
from django.core.exceptions import PermissionDenied
from django.http import HttpResponse
from django.test import RequestFactory
from django.test.utils import override_settings
from django.urls import reverse

from crashlog.middleware import CrashLogMiddleware, process_exception
from crashlog.models import Error


@pytest.mark.django_db
def test_return_value():
    m = CrashLogMiddleware()
    factory = RequestFactory()
    request = factory.get("/my/home/url")
    ret = m.process_exception(request, PermissionDenied())
    assert isinstance(ret, HttpResponse) or ret is None


@pytest.mark.django_db
def test_log():
    m = CrashLogMiddleware()
    factory = RequestFactory()
    request = factory.get("/my/home/url")
    request.user = AnonymousUser()
    request.session = {}
    m.process_exception(request, PermissionDenied())
    assert Error.objects.filter(class_name='PermissionDenied').exists()


@pytest.mark.django_db
def test_log_user():
    m = CrashLogMiddleware()
    factory = RequestFactory()
    request = factory.get("/my/home/url")
    request.user = User.objects.create(username='sax')
    request.session = {}
    m.process_exception(request, PermissionDenied())
    assert Error.objects.filter(class_name='PermissionDenied', username='sax').exists()


@pytest.mark.django_db
def test_ignore_class():
    m = CrashLogMiddleware()
    factory = RequestFactory()
    request = factory.get("/my/home/url")
    request.session = {}

    with override_settings(CRASHLOG_IGNORE_EXCEPTIONS=['django.core.exceptions.PermissionDenied']):
        m.process_exception(request, PermissionDenied())
        assert not Error.objects.filter(class_name='PermissionDenied').exists()


@pytest.mark.django_db
def test_view(django_app, admin_user):
    url = reverse('raise-error')
    with pytest.raises(IndexError):
        res = django_app.get(url, user=admin_user)
        assert res.status_code == 500

    assert Error.objects.filter(class_name='IndexError').exists()


@pytest.mark.django_db
def test_message_user(django_app, admin_user):
    url = reverse('raise_with_message_user')
    res = django_app.get(url, user=admin_user.username)
    assert res.status_code == 200

    assert Error.objects.filter(class_name='IndexError').exists()


@pytest.mark.django_db
def test_simple(rf, django_app, admin_user):
    error = process_exception(PermissionDenied())
    assert error.pk



SITE_ID = 1
STATIC_URL = '/static/'
ROOT_URLCONF = 'demoproject.urls'
SECRET_KEY = ';klkj;okj;lkn;lklj;lkj;kjmlliuewhy2ioqwjdkh'
ALLOWED_HOSTS = ('127.0.0.1',)
PASSWORD_HASHERS = ('django.contrib.auth.hashers.MD5PasswordHasher',)

DATABASES = {
    'default': {
        'ENGINE': 'django.db.backends.postgresql_psycopg2',
        'NAME': 'crashlog',
        'HOST': '127.0.0.1',
        'PORT': '',
        'USER': 'postgres',
        'PASSWORD': ''
    }
}

INSTALLED_APPS = (
    'django.contrib.auth',
    'django.contrib.contenttypes',
    'django.contrib.sessions',
    'django.contrib.sites',
    'django.contrib.admin',
    'admin_extra_urls',
    'crashlog',
)


MIDDLEWARE = (
    'django.middleware.common.CommonMiddleware',
    'django.contrib.sessions.middleware.SessionMiddleware',
    'django.middleware.csrf.CsrfViewMiddleware',
    'django.contrib.auth.middleware.AuthenticationMiddleware',
    'django.contrib.messages.middleware.MessageMiddleware',
    'crashlog.middleware.CrashLogMiddleware',
)


LOGGING = {
    'version': 1,
    'disable_existing_loggers': False,
    'formatters': {
        'simple': {
            'format': '%(levelname)-8s: %(asctime)s %(name)10s: %(funcName)40s %(message)s'
        }
    },
    'handlers': {
        'null': {
            'level': 'DEBUG',
            'class': 'logging.NullHandler'
        },
        'console': {
            'level': 'DEBUG',
            'class': 'logging.StreamHandler',
            'formatter': 'simple'
        },
    },
    'loggers': {
        '': {
            'handlers': ['null'],
            'propagate': True,
            'level': 'DEBUG'
        },
    }
}

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'DIRS': [],
        'APP_DIRS': True,
        'OPTIONS': {
            'context_processors': ['django.contrib.messages.context_processors.messages',
                                   'django.contrib.auth.context_processors.auth',
                                   "django.template.context_processors.request",
                                   ]
        },
    },
]



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='Error',
            fields=[
                ('id', models.AutoField(serialize=False, auto_created=True, primary_key=True, verbose_name='ID')),
                ('class_name', models.CharField(max_length=128, verbose_name='type')),
                ('message', models.TextField()),
                ('traceback', models.TextField()),
                ('date_time', models.DateTimeField(auto_now_add=True)),
                ('url', models.URLField(null=True, blank=True)),
                ('server_name', models.CharField(db_index=True, max_length=128)),
                ('username', models.CharField(blank=True, db_index=True, null=True, max_length=100)),
            ],
            options={
            },
            bases=(models.Model,),
        ),
    ]



# -*- coding: utf-8 -*-

import logging

import pytest
from django.urls import reverse

from crashlog.models import Error

logger = logging.getLogger(__name__)


@pytest.mark.django_db
def test_changelist(django_app, admin_user):
    url = reverse('admin:crashlog_error_changelist')
    res = django_app.get(url, user=admin_user)
    assert res.status_code == 200


@pytest.mark.django_db
def test_display(django_app, admin_user, error):
    url = reverse('admin:crashlog_error_change', args=[error.pk])
    res = django_app.get(url, user=admin_user)
    assert res.status_code == 200


@pytest.mark.django_db
def test_empty_log(django_app, admin_user, error):
    url = reverse('admin:crashlog_error_empty_log')
    res = django_app.get(url, user=admin_user)
    res = res.form.submit()
    assert res.status_code == 302
    assert not Error.objects.filter(id=error.id).exists()




#!/usr/bin/env python
import os
import sys

try:
    me = os.path.realpath(os.readlink(__file__))
except OSError:
    me = os.path.realpath(__file__)
DEMO = os.path.dirname(me)
ROOT = os.path.normpath(os.path.join(DEMO, os.pardir, os.pardir))
TESTS = os.path.normpath(os.path.join(ROOT, 'tests'))
SRC = os.path.normpath(os.path.join(ROOT, 'src'))

sys.path.insert(0, SRC)
if __name__ == '__main__':
    os.environ.setdefault('DJANGO_SETTINGS_MODULE', 'demoproject.settings')
    import logging

    l = logging.getLogger('raven.contrib.django.client.DjangoClient')

    from django.core.management import execute_from_command_line

    debug_on_error = '--pdb' in sys.argv
    args = [a for a in sys.argv if a != '--pdb']

    try:
        execute_from_command_line(args)
    except:
        if debug_on_error:
            import pdb, traceback
            type, value, tb = sys.exc_info()
            traceback.print_exc()
            pdb.post_mortem(tb)
        else:
            raise
